#include "Keyboard.h"

#define KEYBOARD_FUNCTION
#define SERIAL_OUTPUT
#define BUTTONS_COUNT      6
#define LAYERS_COUNT       2

#define BTN_1              8
#define BTN_2              9
#define BTN_3             10
#define BTN_4             11
#define BTN_5             12
#define BTN_6             13


uint8_t currentState[BUTTONS_COUNT];
uint8_t previousState[BUTTONS_COUNT];

typedef enum _ButtonCustomRole_e
{
  NONE,
  LAYER_1,
  LAYER_2
} ButtonCustomRole_e;

typedef struct _ButtonRole_t {
  ButtonCustomRole_e customRole;
  bool shift;
  bool ctrl;
  bool alt;
  char letter;
} ButtonRole_t;

ButtonRole_t buttonsRoles[LAYERS_COUNT][BUTTONS_COUNT] = {
  {
    (ButtonRole_t){.customRole = NONE   , .shift = false, .ctrl = true , .alt = false, .letter = 0},
    (ButtonRole_t){.customRole = NONE   , .shift = false, .ctrl = false, .alt = true , .letter = 0},
    (ButtonRole_t){.customRole = NONE   , .shift = true , .ctrl = false, .alt = false, .letter = 0},
    (ButtonRole_t){.customRole = NONE   , .shift = false, .ctrl = false, .alt = false, .letter = 's'},
    (ButtonRole_t){.customRole = NONE   , .shift = false, .ctrl = false, .alt = true , .letter = 'd'},
    (ButtonRole_t){.customRole = LAYER_1, .shift = false, .ctrl = false, .alt = false, .letter = 0},
  },
  {
    (ButtonRole_t){.customRole = NONE   , .shift = false, .ctrl = false, .alt = false, .letter = 0},
    (ButtonRole_t){.customRole = NONE   , .shift = false, .ctrl = false, .alt = false, .letter = 0},
    (ButtonRole_t){.customRole = NONE   , .shift = false, .ctrl = false, .alt = false, .letter = 0},
    (ButtonRole_t){.customRole = NONE   , .shift = false, .ctrl = false, .alt = false, .letter = 'u'},
    (ButtonRole_t){.customRole = NONE   , .shift = false, .ctrl = false, .alt = true , .letter = 's'},
    (ButtonRole_t){.customRole = NONE   , .shift = false, .ctrl = false, .alt = false, .letter = 0},
  }
};

int i;
volatile bool TimedOut = false;


void SetupTimer();
void StartTimer();
void StopTimer();
void UpdateState(uint8_t* state);
void ActOnState();


void setup() {
  pinMode(BTN_1, INPUT);
  pinMode(BTN_2, INPUT);
  pinMode(BTN_3, INPUT);
  pinMode(BTN_4, INPUT);
  pinMode(BTN_5, INPUT);
  pinMode(BTN_6, INPUT);
  Serial.begin(115200);
  SetupTimer();
  Keyboard.begin();
}

void loop() {
  memset(currentState, 0, sizeof(currentState)/sizeof(currentState[0]));
  UpdateState(currentState);

  ActOnState();

  memcpy(previousState, currentState, sizeof(previousState)/sizeof(previousState[0]));
  delay(10);
}




/*
 * Timer1 is used by Servo lib and Timer2 by tone()
 *
 * Timer1 ticks 62500 times with 16MHz/256
 * 34286
 */
void SetupTimer()
{
  noInterrupts();           // disable all interrupts
  TCCR1A = 0;               // disconnected from attached pins, normal timer (no PWM)
  TCCR1B = 0;               // no clock source (timer stopped)

  TCNT1 = 50000;             // preload timer 65536-16MHz/256 so it ticks 1Hz
  TIMSK1 |= (1 << TOIE1);   // enable timer overflow interrupt
  interrupts();             // enable all interrupts
}

ISR(TIMER1_OVF_vect)        // interrupt service routine that wraps a user defined function supplied by attachInterrupt
{
  StopTimer();
  TCNT1 = 50000;             // preload timer
  TimedOut = true;
}

void StartTimer()
{
  TimedOut = false;
  TCCR1B |= (1 << CS12);    // 256 prescaler
}

void StopTimer()
{
  TCCR1B = 0;               // no clock source (timer stopped)
}

/*
 * This function needs for the pins to be in sequence (one after another).
 */
void UpdateState(uint8_t* state)
{
  for(int i = BTN_1; i < (BTN_1 + BUTTONS_COUNT); i++){
    if (digitalRead(i) == HIGH) {
      state[i - BTN_1] += 1;
    }
  }
}

void ActOnState()
{
  int8_t _layerButton = -1;
  ButtonCustomRole_e _layer = NONE;

  for(int i = 0; i < LAYERS_COUNT; i++){
    for(int j = 0; j < BUTTONS_COUNT; j++){
      ButtonRole_t _btn = buttonsRoles[i][j];
      if (_btn.customRole != NONE) {
        _layerButton = j;
        _layer = _btn.customRole;
      }
    }
  }

  if (digitalRead(BTN_6) == LOW) {
    _layer = NONE;
  }


  switch (_layer) {
      case NONE: {
        /*
        for (uint8_t i = 0; i < BUTTONS_COUNT; i++) {
          if (currentState[i] == 1 && previousState[i] == 0) {
#ifdef SERIAL_OUTPUT
            Serial.print("ON     ");
            Serial.println(i);
#endif
#ifdef KEYBOARD_FUNCTION
            if (buttonsRoles[(uint8_t)_layer][0].shift) {
              Keyboard.press(KEY_LEFT_SHIFT);
            }
            if (buttonsRoles[(uint8_t)_layer][0].ctrl) {
              Keyboard.press(KEY_LEFT_CTRL);
            }
            if (buttonsRoles[(uint8_t)_layer][0].alt) {
              Keyboard.press(KEY_LEFT_ALT);
            }
            if (buttonsRoles[(uint8_t)_layer][0].letter != 0) {
              Keyboard.print(buttonsRoles[(uint8_t)_layer][0].letter);
            }
#endif
          }
          else if (currentState[0] == 0 && previousState[0] > 0) {
#ifdef SERIAL_OUTPUT
            Serial.print("OFF    ");
            Serial.println(i);
#endif
#ifdef KEYBOARD_FUNCTION
            Keyboard.releaseAll();
#endif
          }
        }
        */

        if (currentState[0] == 1 && previousState[0] == 0) {
          Serial.println("ON       1 (P8)");
#ifdef KEYBOARD_FUNCTION
          if (buttonsRoles[(uint8_t)_layer][0].shift) {
            Keyboard.press(KEY_LEFT_SHIFT);
          }
          if (buttonsRoles[(uint8_t)_layer][0].ctrl) {
            Keyboard.press(KEY_LEFT_CTRL);
          }
          if (buttonsRoles[(uint8_t)_layer][0].alt) {
            Keyboard.press(KEY_LEFT_ALT);
          }
          if (buttonsRoles[(uint8_t)_layer][0].letter != 0) {
            Keyboard.print(buttonsRoles[(uint8_t)_layer][0].letter);
          }
#endif
        }
        else if (currentState[0] == 0 && previousState[0] > 0) {
          Serial.println("OFF      1 (P8)");
#ifdef KEYBOARD_FUNCTION
          Keyboard.releaseAll();
#endif
        }


        if (currentState[1] == 1 && previousState[1] == 0) {
          Serial.println("ON       2 (P9)");
#ifdef KEYBOARD_FUNCTION
          if (buttonsRoles[(uint8_t)_layer][1].shift) {
            Keyboard.press(KEY_LEFT_SHIFT);
          }
          if (buttonsRoles[(uint8_t)_layer][1].ctrl) {
            Keyboard.press(KEY_LEFT_CTRL);
          }
          if (buttonsRoles[(uint8_t)_layer][1].alt) {
            Keyboard.press(KEY_LEFT_ALT);
          }
          if (buttonsRoles[(uint8_t)_layer][1].letter != 0) {
            Keyboard.print(buttonsRoles[(uint8_t)_layer][1].letter);
          }
#endif
        }
        else if (currentState[1] == 0 && previousState[1] > 0) {
          Serial.println("OFF      2 (P9)");
#ifdef KEYBOARD_FUNCTION
          Keyboard.releaseAll();
#endif
        }


        if (currentState[2] == 1 && previousState[2] == 0) {
          Serial.println("ON       3 (P10)");
#ifdef KEYBOARD_FUNCTION
          if (buttonsRoles[(uint8_t)_layer][2].shift) {
            Keyboard.press(KEY_LEFT_SHIFT);
          }
          if (buttonsRoles[(uint8_t)_layer][2].ctrl) {
            Keyboard.press(KEY_LEFT_CTRL);
          }
          if (buttonsRoles[(uint8_t)_layer][2].alt) {
            Keyboard.press(KEY_LEFT_ALT);
          }
          if (buttonsRoles[(uint8_t)_layer][2].letter != 0) {
            Keyboard.print(buttonsRoles[(uint8_t)_layer][2].letter);
          }
#endif
        }
        else if (currentState[2] == 0 && previousState[2] > 0) {
          Serial.println("OFF      3 (P10)");
#ifdef KEYBOARD_FUNCTION
          Keyboard.releaseAll();
#endif
        }


        if (currentState[3] == 1 && previousState[3] == 0) {
          Serial.println("ON       4 (P11)");
#ifdef KEYBOARD_FUNCTION
          if (buttonsRoles[(uint8_t)_layer][3].shift) {
            Keyboard.press(KEY_LEFT_SHIFT);
          }
          if (buttonsRoles[(uint8_t)_layer][3].ctrl) {
            Keyboard.press(KEY_LEFT_CTRL);
          }
          if (buttonsRoles[(uint8_t)_layer][3].alt) {
            Keyboard.press(KEY_LEFT_ALT);
          }
          if (buttonsRoles[(uint8_t)_layer][3].letter != 0) {
            Keyboard.print(buttonsRoles[(uint8_t)_layer][3].letter);
          }
#endif
        }
        else if (currentState[3] == 0 && previousState[3] > 0) {
          Serial.println("OFF      4 (P11)");
#ifdef KEYBOARD_FUNCTION
          Keyboard.releaseAll();
#endif
        }


        if (currentState[4] == 1 && previousState[4] == 0) {
          Serial.println("ON       5 (P12)");
#ifdef KEYBOARD_FUNCTION
          if (buttonsRoles[(uint8_t)_layer][4].shift) {
            Keyboard.press(KEY_LEFT_SHIFT);
          }
          if (buttonsRoles[(uint8_t)_layer][4].ctrl) {
            Keyboard.press(KEY_LEFT_CTRL);
          }
          if (buttonsRoles[(uint8_t)_layer][4].alt) {
            Keyboard.press(KEY_LEFT_ALT);
          }
          if (buttonsRoles[(uint8_t)_layer][4].letter != 0) {
            Keyboard.print(buttonsRoles[(uint8_t)_layer][4].letter);
          }
#endif
        }
        else if (currentState[4] == 0 && previousState[4] > 0) {
          Serial.println("OFF      5 (P12)");
#ifdef KEYBOARD_FUNCTION
          Keyboard.releaseAll();
#endif
        }

      } break;



      case LAYER_1: {

        if (currentState[3] == 1 && previousState[3] == 0) {
          Serial.println("ON       4 (P11)");
#ifdef KEYBOARD_FUNCTION
          if (buttonsRoles[(uint8_t)_layer][3].shift) {
            Keyboard.press(KEY_LEFT_SHIFT);
          }
          if (buttonsRoles[(uint8_t)_layer][3].ctrl) {
            Keyboard.press(KEY_LEFT_CTRL);
          }
          if (buttonsRoles[(uint8_t)_layer][3].alt) {
            Keyboard.press(KEY_LEFT_ALT);
          }
          if (buttonsRoles[(uint8_t)_layer][3].letter != 0) {
            Keyboard.print(buttonsRoles[(uint8_t)_layer][3].letter);
          }
#endif
        }
        else if (currentState[3] == 0 && previousState[3] > 0) {
          Serial.println("OFF      4 (P11)");
#ifdef KEYBOARD_FUNCTION
          Keyboard.releaseAll();
#endif
        }


        if (currentState[4] == 1 && previousState[4] == 0) {
          Serial.println("ON       5 (P12)");
#ifdef KEYBOARD_FUNCTION
          if (buttonsRoles[(uint8_t)_layer][4].shift) {
            Keyboard.press(KEY_LEFT_SHIFT);
          }
          if (buttonsRoles[(uint8_t)_layer][4].ctrl) {
            Keyboard.press(KEY_LEFT_CTRL);
          }
          if (buttonsRoles[(uint8_t)_layer][4].alt) {
            Keyboard.press(KEY_LEFT_ALT);
          }
          if (buttonsRoles[(uint8_t)_layer][4].letter != 0) {
            Keyboard.print(buttonsRoles[(uint8_t)_layer][4].letter);
          }
#endif
        }
        else if (currentState[4] == 0 && previousState[4] > 0) {
          Serial.println("OFF      5 (P12)");
#ifdef KEYBOARD_FUNCTION
          Keyboard.releaseAll();
#endif
        }

      } break;

      default: break;
  }
}

